@@ -1,44 +1,51 @@
-#' Do arbitrary operations on a tbl.
+#' Do anything
 #'
-#' This is a general purpose complement to the specialised manipulation
-#' functions \code{\link{filter}}, \code{\link{select}}, \code{\link{mutate}},
-#' \code{\link{summarise}} and \code{\link{arrange}}. You can use \code{do}
+#' \Sexpr[results=rd, stage=render]{dplyr:::lifecycle("questioning")}
+#'
+#' @description This is a general purpose complement to the specialised
+#' manipulation functions [filter()], [select()], [mutate()],
+#' [summarise()] and [arrange()]. You can use `do()`
 #' to perform arbitrary computation, returning either a data frame or
 #' arbitrary objects which will be stored in a list. This is particularly
 #' useful when working with models: you can fit models per group with
-#' \code{do} and then flexibly extract components with either another
-#' \code{do} or \code{summarise}.
+#' `do()` and then flexibly extract components with either another
+#' `do()` or `summarise()`.
 #'
 #' For an empty data frame, the expressions will be evaluated once, even in the
 #' presence of a grouping.  This makes sure that the format of the resulting
 #' data frame is the same for both empty and non-empty input.
 #'
+#' @section Alternative:
+#'
+#' `do()` is marked as questioning as of dplyr 0.8.0, and may be advantageously
+#' replaced by [group_map()].
+#'
 #' @section Connection to plyr:
 #'
-#' If you're familiar with plyr, \code{do} with named arguments is basically
-#' equivalent to \code{dlply}, and \code{do} with a single unnamed argument
-#' is basically equivalent to \code{ldply}. However, instead of storing
+#' If you're familiar with plyr, `do()` with named arguments is basically
+#' equivalent to [plyr::dlply()], and `do()` with a single unnamed argument
+#' is basically equivalent to [plyr::ldply()]. However, instead of storing
 #' labels in a separate attribute, the result is always a data frame. This
-#' means that \code{summarise} applied to the result of \code{do} can
-#' act like \code{ldply}.
+#' means that `summarise()` applied to the result of `do()` can
+#' act like `ldply()`.
 #'
 #' @inheritParams filter
 #' @param .data a tbl
 #' @param ... Expressions to apply to each group. If named, results will be
 #'   stored in a new column. If unnamed, should return a data frame. You can
-#'   use \code{.} to refer to the current group. You can not mix named and
+#'   use `.` to refer to the current group. You can not mix named and
 #'   unnamed arguments.
 #' @return
-#' \code{do} always returns a data frame. The first columns in the data frame
-#' will be the labels, the others will be computed from \code{...}. Named
+#' `do()` always returns a data frame. The first columns in the data frame
+#' will be the labels, the others will be computed from `...`. Named
 #' arguments become list-columns, with one element for each group; unnamed
 #' elements must be data frames and labels will be duplicated accordingly.
 #'
 #' Groups are preserved for a single unnamed input. This is different to
-#' \code{\link{summarise}} because \code{do} generally does not reduce the
+#' [summarise()] because `do()` generally does not reduce the
 #' complexity of the data, it just expresses it in a special way. For
 #' multiple named inputs, the output is grouped by row with
-#' \code{\link{rowwise}}. This allows other verbs to work in an intuitive
+#' [rowwise()]. This allows other verbs to work in an intuitive
 #' way.
 #' @export
 #' @examples
@@ -81,27 +88,51 @@
 #' }
 #' }
 do <- function(.data, ...) {
-  do_(.data, .dots = lazyeval::lazy_dots(...))
+  UseMethod("do")
 }
-
 #' @export
-#' @rdname do
-do_ <- function(.data, ..., .dots) {
+do.default <- function(.data, ...) {
+  do_(.data, .dots = compat_as_lazy_dots(...))
+}
+#' @export
+#' @rdname se-deprecated
+do_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "do_() is deprecated. ",
+    "Please use group_map() instead"
+  ))
   UseMethod("do_")
 }
 
 #' @export
-do_.NULL <- function(.data, ..., .dots) {
+do.NULL <- function(.data, ...) {
+  NULL
+}
+#' @export
+do_.NULL <- function(.data, ..., .dots = list()) {
   NULL
 }
 
 # Helper functions -------------------------------------------------------------
 
-label_output_dataframe <- function(labels, out, groups) {
+env_bind_do_pronouns <- function(env, data) {
+  if (is_function(data)) {
+    bind <- env_bind_active
+  } else {
+    bind <- env_bind
+  }
+
+  # Use `:=` for `.` to avoid partial matching with `.env`
+  bind(env, "." := data, .data = data)
+}
+
+label_output_dataframe <- function(labels, out, groups, .drop) {
   data_frame <- vapply(out[[1]], is.data.frame, logical(1))
   if (any(!data_frame)) {
-    stop("Results are not data frames at positions: ",
-      paste(which(!data_frame), collapse = ", "), call. = FALSE)
+    bad("Results {bad} must be data frames, not {first_bad_class}",
+      bad = fmt_comma(which(!data_frame)),
+      first_bad_class = fmt_classes(out[[1]][[which.min(data_frame)]])
+    )
   }
 
   rows <- vapply(out[[1]], nrow, numeric(1))
@@ -115,7 +146,7 @@ label_output_dataframe <- function(labels, out, groups) {
     labels <- labels[rep(seq_len(nrow(labels)), rows), , drop = FALSE]
     rownames(labels) <- NULL
 
-    grouped_df(bind_cols(labels, out), groups)
+    grouped_df(bind_cols(labels, out), groups, .drop)
   } else {
     rowwise(out)
   }
@@ -136,17 +167,15 @@ named_args <- function(args) {
   # Arguments must either be all named or all unnamed.
   named <- sum(names2(args) != "")
   if (!(named == 0 || named == length(args))) {
-    stop("Arguments to do() must either be all named or all unnamed",
-      call. = FALSE)
+    abort("Arguments must either be all named or all unnamed")
   }
   if (named == 0 && length(args) > 1) {
-    stop("Can only supply single unnamed argument to do()", call. = FALSE)
+    bad("Can only supply one unnamed argument, not {length(args)}")
   }
 
   # Check for old syntax
   if (named == 1 && names(args) == ".f") {
-    stop("do syntax changed in dplyr 0.2. Please see documentation for details",
-      call. = FALSE)
+    abort("do syntax changed in dplyr 0.2. Please see documentation for details")
   }
 
   named != 0
