@@ -2,6 +2,7 @@
 #'
 #' Generate a unique id for each group
 #'
+#' @family grouping functions
 #' @keywords internal
 #' @seealso [group_by()]
 #' @param .data a tbl
@@ -15,21 +16,30 @@ group_indices <- function(.data, ...) {
 }
 #' @export
 group_indices.default <- function(.data, ...) {
-  group_indices_(.data, .dots = compat_as_lazy_dots(...))
+  if (missing(.data)) {
+    rep.int(from_context("..group_number"), from_context("..group_size"))
+  } else {
+    group_indices_(.data, .dots = compat_as_lazy_dots(...))
+  }
 }
 #' @export
 #' @rdname se-deprecated
 group_indices_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "group_indices_() is deprecated. ",
+    "Please use group_indices() instead"
+  ))
+
   UseMethod("group_indices_")
 }
 
 #' @export
-group_indices.data.frame <- function(.data, ...) {
+group_indices.data.frame <- function(.data, ..., .drop = FALSE) {
   dots <- quos(...)
   if (length(dots) == 0L) {
     return(rep(1L, nrow(.data)))
   }
-  grouped_indices_grouped_df_impl(group_by(.data, !!!dots))
+  grouped_indices_grouped_df_impl(group_by(.data, !!!dots, .drop = .drop))
 }
 #' @export
 group_indices_.data.frame <- function(.data, ..., .dots = list()) {
@@ -37,9 +47,23 @@ group_indices_.data.frame <- function(.data, ..., .dots = list()) {
   group_indices(.data, !!!dots)
 }
 
+#' @export
+group_indices.rowwise_df <- function(.data, ...) {
+  if (dots_n(...)) {
+    warn("group_indices_.rowwise_df ignores extra arguments")
+  }
+  seq_len(nrow(.data))
+}
+#' @export
+group_indices_.rowwise_df <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  group_indices(.data, !!!dots)
+}
+
+#' @importFrom rlang dots_n
 #' @export
 group_indices.grouped_df <- function(.data, ...) {
-  if (length(list(...))) {
+  if (dots_n(...)) {
     warn("group_indices_.grouped_df ignores extra arguments")
   }
   grouped_indices_grouped_df_impl(.data)
