@@ -1,6 +1,6 @@
 #' Return rows with matching conditions
 #'
-#' Use `filter()` find rows/cases where conditions are true. Unlike
+#' Use `filter()` to choose rows/cases where conditions are true. Unlike
 #' base subsetting with `[`, rows where the condition evaluates to `NA` are
 #' dropped.
 #'
@@ -15,6 +15,32 @@
 #' * [is.na()]
 #' * [between()], [near()]
 #'
+#' @section Grouped tibbles:
+#'
+#' Because filtering expressions are computed within groups, they may
+#' yield different results on grouped tibbles. This will be the case
+#' as soon as an aggregating, lagging, or ranking function is
+#' involved. Compare this ungrouped filtering:
+#'
+#' ```
+#' starwars %>% filter(mass > mean(mass, na.rm = TRUE))
+#' ```
+#'
+#' With the grouped equivalent:
+#'
+#' ```
+#' starwars %>% group_by(gender) %>% filter(mass > mean(mass, na.rm = TRUE))
+#' ```
+#'
+#' The former keeps rows with `mass` greater than the global average
+#' whereas the latter keeps rows with `mass` greater than the gender
+#' average.
+#'
+#' It is valid to use grouping variables in filter expressions.
+#'
+#' When applied on a grouped tibble, `filter()` automatically [rearranges][arrange]
+#' the tibble by groups for performance reasons.
+#'
 #' @section Tidy data:
 #' When applied to a data frame, row names are silently dropped. To preserve,
 #' convert to an explicit variable with [tibble::rownames_to_column()].
@@ -31,11 +57,13 @@
 #'   Multiple conditions are combined with `&`. Only rows where the
 #'   condition evaluates to `TRUE` are kept.
 #'
-#'   These arguments are automatically [quoted][rlang::quo] and
+#'   The arguments in `...` are automatically [quoted][rlang::quo] and
 #'   [evaluated][rlang::eval_tidy] in the context of the data
 #'   frame. They support [unquoting][rlang::quasiquotation] and
 #'   splicing. See `vignette("programming")` for an introduction to
 #'   these concepts.
+#' @param .preserve when `FALSE` (the default), the grouping structure
+#'   is recalculated based on the resulting data, otherwise it is kept as is.
 #' @return An object of the same class as `.data`.
 #' @seealso [filter_all()], [filter_if()] and [filter_at()].
 #' @export
@@ -49,33 +77,75 @@
 #'
 #' # Multiple arguments are equivalent to and
 #' filter(starwars, hair_color == "none", eye_color == "black")
-filter <- function(.data, ...) {
+#'
+#'
+#' # The filtering operation may yield different results on grouped
+#' # tibbles because the expressions are computed within groups.
+#' #
+#' # The following filters rows where `mass` is greater than the
+#' # global average:
+#' starwars %>% filter(mass > mean(mass, na.rm = TRUE))
+#'
+#' # Whereas this keeps rows with `mass` greater than the gender
+#' # average:
+#' starwars %>% group_by(gender) %>% filter(mass > mean(mass, na.rm = TRUE))
+#'
+#'
+#' # Refer to column names stored as strings with the `.data` pronoun:
+#' vars <- c("mass", "height")
+#' cond <- c(80, 150)
+#' starwars %>%
+#'   filter(
+#'     .data[[vars[[1]]]] > cond[[1]],
+#'     .data[[vars[[2]]]] > cond[[2]]
+#'   )
+#'
+#' # For more complex cases, knowledge of tidy evaluation and the
+#' # unquote operator `!!` is required. See https://tidyeval.tidyverse.org/
+#' #
+#' # One useful and simple tidy eval technique is to use `!!` to bypass
+#' # the data frame and its columns. Here is how to filter the columns
+#' # `mass` and `height` relative to objects of the same names:
+#' mass <- 80
+#' height <- 150
+#' filter(starwars, mass > !!mass, height > !!height)
+filter <- function(.data, ..., .preserve = FALSE) {
   UseMethod("filter")
 }
 #' @export
-filter.default <- function(.data, ...) {
+filter.default <- function(.data, ..., .preserve = FALSE) {
   filter_(.data, .dots = compat_as_lazy_dots(...))
 }
 #' @export
 #' @rdname se-deprecated
 filter_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "filter_() is deprecated. ",
+    "Please use filter() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with filter() : https://tidyeval.tidyverse.org"
+  ))
   UseMethod("filter_")
 }
 
-#' Select rows by position
+#' Choose rows by position
+#'
+#' Choose rows by their ordinal position in the tbl.  Grouped tbls use
+#' the ordinal position within the group.
 #'
 #' Slice does not work with relational databases because they have no
 #' intrinsic notion of row order. If you want to perform the equivalent
 #' operation, use [filter()] and [row_number()].
 #'
-#' Positive values select rows to keep; negative values drop rows. The
-#' values provided must be either all positive or all negative.
-#'
 #' @family single table verbs
 #' @param .data A tbl.
-#' @param ... Integer row values.
+#' @param ... Integer row values.  Provide either positive values to keep,
+#'   or negative values to drop. The values provided must be either all
+#'   positive or all negative.  Indices beyond the number of rows in the
+#'   input are silently ignored.
 #'
-#'   These arguments are automatically [quoted][rlang::quo] and
+#'   The arguments in `...` are automatically [quoted][rlang::quo] and
 #'   [evaluated][rlang::eval_tidy] in the context of the data
 #'   frame. They support [unquoting][rlang::quasiquotation] and
 #'   splicing. See `vignette("programming")` for an introduction to
@@ -85,6 +155,7 @@ filter_ <- function(.data, ..., .dots = list()) {
 #' @export
 #' @examples
 #' slice(mtcars, 1L)
+#' # Similar to tail(mtcars, 1):
 #' slice(mtcars, n())
 #' slice(mtcars, 5:n())
 #' # Rows can be dropped with negative indices:
@@ -101,23 +172,33 @@ filter_ <- function(.data, ..., .dots = list()) {
 #' filter(mtcars, row_number() == 1L)
 #' filter(mtcars, row_number() == n())
 #' filter(mtcars, between(row_number(), 5, n()))
-slice <- function(.data, ...) {
+slice <- function(.data, ..., .preserve = FALSE) {
   UseMethod("slice")
 }
 #' @export
-slice.default <- function(.data, ...) {
+slice.default <- function(.data, ..., .preserve = FALSE) {
   slice_(.data, .dots = compat_as_lazy_dots(...))
 }
 #' @export
 #' @rdname se-deprecated
 slice_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "slice_() is deprecated. ",
+    "Please use slice() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with slice() : https://tidyeval.tidyverse.org"
+  ))
   UseMethod("slice_")
 }
 
-#' Reduces multiple values down to a single value
+#' Reduce multiple values down to a single value
 #'
-#' `summarise()` is typically used on grouped data created by [group_by()].
-#' The output will have one row for each group.
+#' Create one or more scalar variables summarizing the variables of an
+#' existing tbl. Tbls with groups created by [group_by()] will result in one
+#' row in the output for each group.  Tbls with no groups will result in one row.
+#'
+#' `summarise()` and `summarize()` are synonyms.
 #'
 #' @section Useful functions:
 #'
@@ -140,7 +221,7 @@ slice_ <- function(.data, ..., .dots = list()) {
 #'   name of the variable in the result. The value should be an expression
 #'   that returns a single value like `min(x)`, `n()`, or `sum(is.na(y))`.
 #'
-#'   These arguments are automatically [quoted][rlang::quo] and
+#'   The arguments in `...` are automatically [quoted][rlang::quo] and
 #'   [evaluated][rlang::eval_tidy] in the context of the data
 #'   frame. They support [unquoting][rlang::quasiquotation] and
 #'   splicing. See `vignette("programming")` for an introduction to
@@ -172,10 +253,18 @@ slice_ <- function(.data, ..., .dots = list()) {
 #'   summarise(disp = mean(disp), sd = sd(disp))
 #'
 #'
-#' # summarise() supports quasiquotation. You can unquote raw
-#' # expressions or quosures:
-#' var <- quo(mean(cyl))
-#' summarise(mtcars, !!var)
+#' # Refer to column names stored as strings with the `.data` pronoun:
+#' var <- "mass"
+#' summarise(starwars, avg = mean(.data[[var]], na.rm = TRUE))
+#'
+#' # For more complex cases, knowledge of tidy evaluation and the
+#' # unquote operator `!!` is required. See https://tidyeval.tidyverse.org/
+#' #
+#' # One useful and simple tidy eval technique is to use `!!` to
+#' # bypass the data frame and its columns. Here is how to divide the
+#' # column `mass` by an object of the same name:
+#' mass <- 100
+#' summarise(starwars, avg = mean(mass / !!mass, na.rm = TRUE))
 summarise <- function(.data, ...) {
   UseMethod("summarise")
 }
@@ -186,6 +275,14 @@ summarise.default <- function(.data, ...) {
 #' @export
 #' @rdname se-deprecated
 summarise_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "summarise_() is deprecated. ",
+    "Please use summarise() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with summarise() : https://tidyeval.tidyverse.org"
+  ))
+
   UseMethod("summarise_")
 }
 
@@ -197,16 +294,16 @@ summarize <- summarise
 summarize_ <- summarise_
 
 
-#' Add new variables
+#' Create or transform variables
 #'
-#' `mutate()` adds new variables and preserves existing;
-#' `transmute()` drops existing variables.
+#' `mutate()` adds new variables and preserves existing ones;
+#' `transmute()` adds new variables and drops existing ones.  Both
+#' functions preserve the number of rows of the input.
+#' New variables overwrite existing variables of the same name.
 #'
-#' @section Useful functions:
+#' @section Useful functions available in calculations of variables:
 #'
-#' * [`+`], [`-`] etc
-#'
-#' * [log()]
+#' * [`+`], [`-`], [log()], etc., for their usual mathematical meanings
 #'
 #' * [lead()], [lag()]
 #'
@@ -219,6 +316,36 @@ summarize_ <- summarise_
 #'
 #' * [if_else()], [recode()], [case_when()]
 #'
+#' @section Grouped tibbles:
+#'
+#' Because mutating expressions are computed within groups, they may
+#' yield different results on grouped tibbles. This will be the case
+#' as soon as an aggregating, lagging, or ranking function is
+#' involved. Compare this ungrouped mutate:
+#'
+#' ```
+#' starwars %>%
+#'   mutate(mass / mean(mass, na.rm = TRUE)) %>%
+#'   pull()
+#' ```
+#'
+#' With the grouped equivalent:
+#'
+#' ```
+#' starwars %>%
+#'   group_by(gender) %>%
+#'   mutate(mass / mean(mass, na.rm = TRUE)) %>%
+#'   pull()
+#' ```
+#'
+#' The former normalises `mass` by the global average whereas the
+#' latter normalises by the averages within gender levels.
+#'
+#' Note that you can't overwrite a grouping variable within
+#' `mutate()`.
+#'
+#' `mutate()` does not evaluate the expressions when the group is empty.
+#'
 #' @section Scoped mutation and transmutation:
 #'
 #' The three [scoped] variants of `mutate()` ([mutate_all()],
@@ -230,10 +357,14 @@ summarize_ <- summarise_
 #' @export
 #' @inheritParams filter
 #' @inheritSection filter Tidy data
-#' @param ... Name-value pairs of expressions. Use `NULL` to drop
-#'   a variable.
+#' @param ... Name-value pairs of expressions, each with length 1 or the same
+#'   length as the number of rows in the group (if using [group_by()]) or in the entire
+#'   input (if not using groups). The name of each argument will be the name of
+#'   a new variable, and the value will be its corresponding value.  Use a `NULL`
+#'   value in `mutate` to drop a variable.  New variables overwrite existing variables
+#'   of the same name.
 #'
-#'   These arguments are automatically [quoted][rlang::quo] and
+#'   The arguments in `...` are automatically [quoted][rlang::quo] and
 #'   [evaluated][rlang::eval_tidy] in the context of the data
 #'   frame. They support [unquoting][rlang::quasiquotation] and
 #'   splicing. See `vignette("programming")` for an introduction to
@@ -274,10 +405,37 @@ summarize_ <- summarise_
 #'   transmute(displ_l = disp / 61.0237)
 #'
 #'
-#' # mutate() supports quasiquotation. You can unquote quosures, which
-#' # can refer to both contextual variables and variable names:
-#' var <- 100
-#' as_tibble(mtcars) %>% mutate(cyl = !!quo(cyl * var))
+#' # The mutate operation may yield different results on grouped
+#' # tibbles because the expressions are computed within groups.
+#' # The following normalises `mass` by the global average:
+#' starwars %>%
+#'   mutate(mass / mean(mass, na.rm = TRUE)) %>%
+#'   pull()
+#'
+#' # Whereas this normalises `mass` by the averages within gender
+#' # levels:
+#' starwars %>%
+#'   group_by(gender) %>%
+#'   mutate(mass / mean(mass, na.rm = TRUE)) %>%
+#'   pull()
+#'
+#' # Note that you can't overwrite grouping variables:
+#' gdf <- mtcars %>% group_by(cyl)
+#' try(mutate(gdf, cyl = cyl * 100))
+#'
+#'
+#' # Refer to column names stored as strings with the `.data` pronoun:
+#' vars <- c("mass", "height")
+#' mutate(starwars, prod = .data[[vars[[1]]]] * .data[[vars[[2]]]])
+#'
+#' # For more complex cases, knowledge of tidy evaluation and the
+#' # unquote operator `!!` is required. See https://tidyeval.tidyverse.org/
+#' #
+#' # One useful and simple tidy eval technique is to use `!!` to
+#' # bypass the data frame and its columns. Here is how to divide the
+#' # column `mass` by an object of the same name:
+#' mass <- 100
+#' mutate(starwars, mass = mass / !!mass)
 mutate <- function(.data, ...) {
   UseMethod("mutate")
 }
@@ -288,6 +446,14 @@ mutate.default <- function(.data, ...) {
 #' @export
 #' @rdname se-deprecated
 mutate_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "mutate_() is deprecated. ",
+    "Please use mutate() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with mutate() : https://tidyeval.tidyverse.org"
+  ))
+
   UseMethod("mutate_")
 }
 
@@ -299,12 +465,19 @@ transmute <- function(.data, ...) {
 #' @rdname se-deprecated
 #' @export
 transmute_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "transmute_() is deprecated. ",
+    "Please use transmute() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with transmute() : https://tidyeval.tidyverse.org"
+  ))
   UseMethod("transmute_")
 }
 
 #' @export
 transmute.default <- function(.data, ...) {
-  dots <- named_quos(...)
+  dots <- quos(..., .named = TRUE)
   out <- mutate(.data, !!!dots)
 
   keep <- names(dots)
@@ -318,7 +491,7 @@ transmute_.default <- function(.data, ..., .dots = list()) {
 
 #' @export
 transmute.grouped_df <- function(.data, ...) {
-  dots <- named_quos(...)
+  dots <- quos(..., .named = TRUE)
   out <- mutate(.data, !!!dots)
   keep <- names(dots)
 
@@ -333,17 +506,22 @@ transmute_.grouped_df <- function(.data, ..., .dots = list()) {
 
 #' Arrange rows by variables
 #'
-#' Use [desc()] to sort a variable in descending order.
+#' Order tbl rows by an expression involving its variables.
 #'
 #' @section Locales:
 #' The sort order for character vectors will depend on the collating sequence
 #' of the locale in use: see [locales()].
 #'
+#' @section Missing values:
+#' Unlike base sorting with `sort()`, `NA` are:
+#' * always sorted to the end for local data, even when wrapped with `desc()`.
+#' * treated differently for remote data, depending on the backend.
+#'
 #' @export
 #' @inheritParams filter
 #' @inheritSection filter Tidy data
-#' @param ... Comma separated list of unquoted variable names. Use
-#'   [desc()] to sort a variable in descending order.
+#' @param ... Comma separated list of unquoted variable names, or expressions
+#'   involving variable names. Use [desc()] to sort a variable in descending order.
 #' @family single table verbs
 #' @return An object of the same class as `.data`.
 #' @examples
@@ -365,6 +543,14 @@ arrange.default <- function(.data, ...) {
 #' @export
 #' @rdname se-deprecated
 arrange_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "arrange_() is deprecated. ",
+    "Please use arrange() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with arrange() : https://tidyeval.tidyverse.org"
+  ))
+
   UseMethod("arrange_")
 }
 
@@ -379,23 +565,31 @@ arrange.grouped_df <- function(.data, ..., .by_group = FALSE) {
     dots <- quos(...)
   }
 
-  arrange_impl(.data, dots)
+  arrange_impl(.data, dots, environment())
 }
 
 #' Select/rename variables by name
 #'
+#' Choose or rename variables from a tbl.
 #' `select()` keeps only the variables you mention; `rename()`
 #' keeps all variables.
 #'
+#' These functions work by column index, not value; thus, an expression
+#' like `select(data.frame(x = 1:5, y = 10), z = x+1)` does not create a variable
+#' with values `2:6`. (In the current implementation, the expression `z = x+1`
+#' wouldn't do anything useful.)  To calculate using column values, see
+#' [mutate()]/[transmute()].
+#'
 #' @section Useful functions:
 #' As well as using existing functions like `:` and `c()`, there are
-#' a number of special functions that only work inside `select`
+#' a number of special functions that only work inside `select()`:
 #'
 #' * [starts_with()], [ends_with()], [contains()]
 #' * [matches()]
 #' * [num_range()]
 #' * [one_of()]
 #' * [everything()]
+#' * [group_cols()]
 #'
 #' To drop variables, use `-`.
 #'
@@ -414,19 +608,23 @@ arrange.grouped_df <- function(.data, ..., .by_group = FALSE) {
 #' @inheritParams filter
 #' @inheritSection filter Tidy data
 #' @param ... One or more unquoted expressions separated by commas.
-#'   You can treat variable names like they are positions.
+#'   You can treat variable names like they are positions, so you can
+#'   use expressions like `x:y` to select ranges of variables.
 #'
-#'   Positive values select variables; negative values to drop variables.
+#'   Positive values select variables; negative values drop variables.
 #'   If the first expression is negative, `select()` will automatically
 #'   start with all variables.
 #'
-#'   Use named arguments to rename selected variables.
+#'   Use named arguments, e.g. `new_name = old_name`, to rename selected variables.
 #'
-#'   These arguments are automatically [quoted][rlang::quo] and
+#'   The arguments in `...` are automatically [quoted][rlang::quo] and
 #'   [evaluated][rlang::eval_tidy] in a context where column names
-#'   represent column positions. They support
+#'   represent column positions. They also support
 #'   [unquoting][rlang::quasiquotation] and splicing. See
 #'   `vignette("programming")` for an introduction to these concepts.
+#'
+#'   See [select helpers][tidyselect::select_helpers] for more details and
+#'   examples about tidyselect helpers such as `starts_with()`, `everything()`, ...
 #' @return An object of the same class as `.data`.
 #' @family single table verbs
 #' @export
@@ -438,6 +636,10 @@ arrange.grouped_df <- function(.data, ..., .by_group = FALSE) {
 #' # Move Species variable to the front
 #' select(iris, Species, everything())
 #'
+#' # Move Sepal.Length variable to back
+#' # first select all variables except Sepal.Length, then re select Sepal.Length
+#' select(iris, -Sepal.Length, Sepal.Length)
+#'
 #' df <- as.data.frame(matrix(runif(100), nrow = 10))
 #' df <- tbl_df(df[c(3, 4, 7, 1, 9, 8, 5, 2, 6, 10)])
 #' select(df, V4:V6)
@@ -446,6 +648,9 @@ arrange.grouped_df <- function(.data, ..., .by_group = FALSE) {
 #' # Drop variables with -
 #' select(iris, -starts_with("Petal"))
 #'
+#' # Select the grouping variables:
+#' starwars %>% group_by(gender) %>% select(group_cols())
+#'
 #'
 #' # The .data pronoun is available:
 #' select(mtcars, .data$cyl)
@@ -463,6 +668,8 @@ arrange.grouped_df <- function(.data, ..., .by_group = FALSE) {
 #' # * rename() keeps all variables
 #' rename(iris, petal_length = Petal.Length)
 #'
+#' # * select() can rename variables in a group
+#' select(iris, obs = starts_with('S'))
 #'
 #' # Unquoting ----------------------------------------
 #'
@@ -489,6 +696,14 @@ select.default <- function(.data, ...) {
 #' @export
 #' @rdname se-deprecated
 select_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "select_() is deprecated. ",
+    "Please use select() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with select() : https://tidyeval.tidyverse.org"
+  ))
+
   UseMethod("select_")
 }
 
@@ -504,6 +719,14 @@ rename.default <- function(.data, ...) {
 #' @rdname se-deprecated
 #' @export
 rename_ <- function(.data, ..., .dots = list()) {
+  signal_soft_deprecated(paste_line(
+    "rename_() is deprecated. ",
+    "Please use rename() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with rename() : https://tidyeval.tidyverse.org"
+  ))
+
   UseMethod("rename_")
 }
 
@@ -522,10 +745,9 @@ rename_ <- function(.data, ..., .dots = list()) {
 #' filter(carriers, n() < 100)
 #' }
 n <- function() {
-  abort("This function should not be called directly")
+  from_context("..group_size")
 }
 
-
 #' Deprecated SE versions of main verbs.
 #'
 #' dplyr used to offer twin versions of each verb suffixed with an
