@@ -61,15 +61,6 @@ CharacterVector strings_addresses(CharacterVector s) {
   return res;
 }
 
-// simple internal debugging function to access the gp part of the SEXP
-// only meant for internal use in dplyr debugging
-
-// [[Rcpp::export]]
-unsigned short gp(SEXP x) {
-  return reinterpret_cast<sxpinfo_struct*>(x)->gp;
-}
-
-
 //' Enable internal logging
 //'
 //' Log entries, depending on the log level, will be printed to the standard
@@ -83,3 +74,20 @@ unsigned short gp(SEXP x) {
 void init_logging(const std::string& log_level) {
   plog::init_r(log_level);
 }
+
+// [[Rcpp::export]]
+bool is_maybe_shared(SEXP env, SEXP name) {
+  SEXP x = Rf_eval(name, env);
+  return MAYBE_SHARED(x);
+}
+
+// [[Rcpp::export]]
+LogicalVector maybe_shared_columns(SEXP df) {
+  int n = Rf_length(df);
+  LogicalVector res(no_init(n));
+  for (int i = 0; i < n; i++) {
+    res[i] = MAYBE_SHARED(VECTOR_ELT(df, i));
+  }
+  return res;
+}
+
