@@ -8,10 +8,11 @@
 #include <tools/pointer_vector.h>
 #include <tools/utils.h>
 
-#include <dplyr/GroupedDataFrame.h>
+#include <dplyr/data/GroupedDataFrame.h>
+#include <dplyr/data/NaturalDataFrame.h>
 #include <dplyr/Collecter.h>
-#include <dplyr/bad.h>
-#include <dplyr/tbl_cpp.h>
+#include <tools/bad.h>
+#include <tools/set_rownames.h>
 
 using namespace Rcpp;
 using namespace dplyr;
@@ -151,8 +152,8 @@ bool dplyr_is_bind_spliceable(SEXP x) {
   if (TYPEOF(x) != VECSXP)
     return false;
 
-  if (OBJECT(x))
-    return Rf_inherits(x, "spliced");
+  if (Rf_inherits(x, "spliced")) return true;
+  if (Rf_inherits(x, "data.frame")) return false;
 
   for (R_xlen_t i = 0; i != Rf_xlength(x); ++i) {
     if (is_atomic(VECTOR_ELT(x, i)))
@@ -181,11 +182,9 @@ List rbind__impl(List dots, const SymbolString& id) {
   R_xlen_t n = 0;
   std::vector<SEXP> chunks;
   std::vector<R_xlen_t> df_nrows;
-  std::vector<String> dots_names;
 
   chunks.reserve(ndata);
   df_nrows.reserve(ndata);
-  dots_names.reserve(ndata);
 
   int k = 0;
   for (int i = 0; i < ndata; i++) {
@@ -195,15 +194,11 @@ List rbind__impl(List dots, const SymbolString& id) {
     R_xlen_t nrows = rows_length(chunks[k], true);
     df_nrows.push_back(nrows);
     n += nrows;
-    if (!id.is_empty()) {
-      dots_names.push_back(name_at(dots, i));
-    }
     k++;
   }
   ndata = chunks.size();
   pointer_vector<Collecter> columns;
 
-
   LOG_VERBOSE << "binding " << ndata << " chunks";
 
   SymbolVector names;
@@ -294,14 +289,37 @@ List rbind__impl(List dots, const SymbolString& id) {
 
   // Add vector of identifiers if .id is supplied
   if (!id.is_empty()) {
-    CharacterVector id_col = no_init(n);
 
-    CharacterVector::iterator it = id_col.begin();
-    for (int i = 0; i < ndata; ++i) {
-      std::fill(it, it + df_nrows[i], dots_names[i]);
-      it += df_nrows[i];
+    // extract the names
+    SEXP dots_names(vec_names(dots));
+    if (Rf_isNull(dots_names)) {
+      out[0] = Rf_allocVector(STRSXP, n);
+    } else {
+      // use the SEXP* directly so that we don't have to pay to check
+      // that dots_names is a STRSXP every single time
+      SEXP* p_dots_names = STRING_PTR(dots_names);
+      SEXP* p_dots = get_vector_ptr(dots);
+
+      // we id_col now, so it is definitely younger than dots_names
+      // this is surely write barrier proof
+      SEXP id_col = PROTECT(Rf_allocVector(STRSXP, n));
+
+      SEXP* p_id_col = STRING_PTR(id_col);
+      for (int i = 0; i < ndata; ++i, ++p_dots_names, ++p_dots) {
+
+        // skip NULL on dots. because the way df_nrows is made above
+        // need to skip dots_names too
+        if (Rf_isNull(*p_dots)) {
+          ++p_dots;
+          ++p_dots_names;
+        }
+
+        p_id_col = std::fill_n(p_id_col, df_nrows[i], *p_dots_names);
+      }
+      out[0] = id_col;
+      UNPROTECT(1);
     }
-    out[0] = id_col;
+
     out_names.set(0, id);
   }
   out.attr("names") = out_names;
@@ -309,20 +327,19 @@ List rbind__impl(List dots, const SymbolString& id) {
 
   LOG_VERBOSE << "result has " << n << " rows";
 
-  // infer the classes and extra info (groups, etc ) from the first (#1692)
+  // infer the classes group info from the first (#1692)
   if (ndata) {
     SEXP first = chunks[0];
     if (Rf_inherits(first, "data.frame")) {
       set_class(out, get_class(first));
-      if (Rf_inherits(first, "grouped_df")) {
-        copy_vars(out, first);
-        out = GroupedDataFrame(out).data();
+      if (is<GroupedDataFrame>(first)) {
+        out = GroupedDataFrame(out, GroupedDataFrame(first)).data();
       }
     } else {
-      set_class(out, classes_not_grouped());
+      set_class(out, NaturalDataFrame::classes());
     }
   } else {
-    set_class(out, classes_not_grouped());
+    set_class(out, NaturalDataFrame::classes());
   }
 
   return out;
@@ -373,7 +390,9 @@ List cbind_all(List dots) {
   // collect columns
   List out(nv);
   CharacterVector out_names(nv);
-  SEXP dots_names = vec_names(dots);
+
+  // Can't use CharacterVector because the result might be R_NilValue
+  RObject dots_names = vec_names(dots);
 
   // then do the subsequent dfs
   for (int i = first_i, k = 0; i < n_dots; i++) {
@@ -402,7 +421,7 @@ List cbind_all(List dots) {
   if (Rf_inherits(first, "data.frame")) {
     copy_most_attributes(out, first);
   } else {
-    set_class(out, classes_not_grouped());
+    set_class(out, NaturalDataFrame::classes());
   }
 
   out.names() = out_names;
