@@ -65,7 +65,7 @@ test_that("arrange handles the case where ... is missing (#338)", {
 
 test_that("arrange handles 0-rows data frames", {
   d <- data.frame(a = numeric(0))
-  expect_equal(d, arrange(d))
+  expect_identical(d, arrange(d))
 })
 
 test_that("grouped arrange ignores group (#491 -> #1206)", {
@@ -76,7 +76,7 @@ test_that("grouped arrange ignores group (#491 -> #1206)", {
 })
 
 test_that("arrange keeps the grouping structure (#605)", {
-  dat <- data_frame(g = c(2, 2, 1, 1), x = c(1, 3, 2, 4))
+  dat <- tibble(g = c(2, 2, 1, 1), x = c(1, 3, 2, 4))
   res <- dat %>% group_by(g) %>% arrange()
   expect_is(res, "grouped_df")
   expect_equal(res$x, dat$x)
@@ -84,7 +84,7 @@ test_that("arrange keeps the grouping structure (#605)", {
   res <- dat %>% group_by(g) %>% arrange(x)
   expect_is(res, "grouped_df")
   expect_equal(res$x, 1:4)
-  expect_equal(attr(res, "indices"), list(c(1, 3), c(0, 2)))
+  expect_equal(group_rows(res), list(c(2, 4), c(1, 3)))
 })
 
 test_that("arrange handles complex vectors", {
@@ -105,14 +105,14 @@ test_that("arrange handles complex vectors", {
   expect_true(all(is.na(res$y[9:10])))
 })
 
-test_that("arrange respects attributes #1105", {
-  env <- environment()
-  Period <- suppressWarnings(setClass("Period", contains = "numeric", where = env))
-  on.exit(removeClass("Period", where = env))
+test_that("arrange respects S4 classes #1105", {
+  TestS4 <- suppressWarnings(setClass("TestS4", contains = "numeric"))
+  setMethod('[', 'TestS4', function(x, i, ...){ TestS4(unclass(x)[i, ...])  })
+  on.exit(removeClass("TestS4"))
 
-  df <- data.frame(p = Period(c(1, 2, 3)), x = 1:3)
+  df <- data.frame(p = TestS4(c(1, 2, 3)), x = 1:3)
   res <- arrange(df, p)
-  expect_is(res$p, "Period")
+  expect_is(res$p, "TestS4")
 })
 
 test_that("arrange works with empty data frame (#1142)", {
@@ -123,7 +123,7 @@ test_that("arrange works with empty data frame (#1142)", {
 })
 
 test_that("arrange respects locale (#1280)", {
-  df2 <- data_frame(words = c("casa", "\u00e1rbol", "zona", "\u00f3rgano"))
+  df2 <- tibble(words = c("casa", "\u00e1rbol", "zona", "\u00f3rgano"))
 
   res <- df2 %>% arrange(words)
   expect_equal(res$words, sort(df2$words))
@@ -158,7 +158,7 @@ test_that("arrange fails gracefully on list columns (#1489)", {
 })
 
 test_that("arrange supports raw columns (#1803)", {
-  df <- data_frame(a = 1:3, b = as.raw(1:3))
+  df <- tibble(a = 1:3, b = as.raw(1:3))
   expect_identical(arrange(df, a), df)
   expect_identical(arrange(df, b), df)
   expect_identical(arrange(df, desc(a)), df[3:1, ])
@@ -166,7 +166,7 @@ test_that("arrange supports raw columns (#1803)", {
 })
 
 test_that("arrange fails gracefully on matrix input (#1870)", {
-  df <- data_frame(a = 1:3, b = 4:6)
+  df <- tibble(a = 1:3, b = 4:6)
   expect_error(
     arrange(df, is.na(df)),
     "Argument 1 is of unsupported type matrix",
@@ -179,6 +179,22 @@ test_that("arrange fails gracefully on data.frame input (#3153)", {
   expect_error(arrange(df, iris), "Argument 1 is of unsupported type data.frame")
 })
 
+test_that("arrange.data.frame recognizes the .by_group argument (#3546)", {
+  df <- data.frame(foo=1:2, bar=2)
+  res <- df %>%
+    arrange(foo, .by_group=TRUE)
+  expect_identical(res, df)
+})
+
+test_that("desc(<not just a symbol>) works (#4099)", {
+  df <- data.frame(x = rep(1, 5), y = c(0, 3, 1.5, -5, 4)) %>%
+    mutate(diff = x - y, absdiff = abs(x - y))
+  expect_identical(
+    arrange(df, desc(abs(diff))),
+    arrange(df, desc(absdiff))
+  )
+})
+
 # grouped_df --------------------------------------------------------------
 
 test_that("can choose to include grouping vars", {
