@@ -3,14 +3,17 @@ context("colwise utils")
 test_that("tbl_at_vars() errs on bad input", {
   expect_error(
     tbl_at_vars(iris, raw(3)),
-    "`.vars` must be a character/numeric vector or a `vars()` object, not raw",
+    "`.vars` must be a character/numeric vector or a `vars()` object, not a raw vector",
     fixed = TRUE
   )
 })
 
 test_that("tbl_at_vars() treats `NULL` as empty inputs", {
   expect_identical(tbl_at_vars(mtcars, vars(NULL)), tbl_at_vars(mtcars, vars()))
-  expect_identical(mutate_at(mtcars, vars(NULL), `*`, 100), mtcars)
+  expect_identical(
+    tibble::remove_rownames(mutate_at(mtcars, vars(NULL), `*`, 100)),
+    tibble::remove_rownames(mtcars)
+  )
 })
 
 test_that("tbl_if_vars() errs on bad input", {
@@ -19,4 +22,12 @@ test_that("tbl_if_vars() errs on bad input", {
     "`.predicate` must have length 1, not 2",
     fixed = TRUE
   )
+
+  .funs <- list(identity, force)
+  .funs <- as_fun_list(.funs, enquo(.funs), caller_env())
+  expect_error(
+    tbl_if_vars(iris, .funs, environment()),
+    "`.predicate` must have length 1, not 2",
+    fixed = TRUE
+  )
 })
